@@ -19,13 +19,13 @@ import copy
 import logging
 import sys
 
+from openstack.config import loader as config   # noqa
+from openstack import connection
 from oslo_utils import strutils
 import six
 
 from osc_lib.api import auth
 from osc_lib import exceptions
-from osc_lib import session as osc_session
-from osc_lib import version
 
 
 LOG = logging.getLogger(__name__)
@@ -56,6 +56,11 @@ class ClientCache(object):
 class ClientManager(object):
     """Manages access to API clients, including authentication."""
 
+    # NOTE(dtroyer): Keep around the auth required state of the _current_
+    #                command since ClientManager has no visibility to the
+    #                command itself; assume auth is not required.
+    _auth_required = False
+
     def __init__(
         self,
         cli_options=None,
@@ -96,46 +101,12 @@ class ClientManager(object):
         # self.verify is the Requests-compatible form
         # self.cacert is the form used by the legacy client libs
         # self.insecure is not needed, use 'not self.verify'
-
-        # NOTE(dtroyer): Per bug https://bugs.launchpad.net/bugs/1447784
-        #                --insecure overrides any --os-cacert setting
-
-        # Set a hard default
-        self.verify = True
-        if self._cli_options.insecure:
-            # Handle --insecure
-            self.verify = False
-            self.cacert = None
-        else:
-            if (self._cli_options.cacert is not None
-                    and self._cli_options.cacert != ''):
-                # --cacert implies --verify here
-                self.verify = self._cli_options.cacert
-                self.cacert = self._cli_options.cacert
-            else:
-                # Fall through also gets --verify
-                if self._cli_options.verify is not None:
-                    self.verify = self._cli_options.verify
-                self.cacert = None
-
-        # Set up client certificate and key
-        # NOTE(cbrandily): This converts client certificate/key to requests
-        #                  cert argument: None (no client certificate), a path
-        #                  to client certificate or a tuple with client
-        #                  certificate/key paths.
-        self.cert = self._cli_options.cert
-        if self.cert and self._cli_options.key:
-            self.cert = self.cert, self._cli_options.key
-
-        # TODO(mordred) The logic above to set all of these is duplicated in
-        # os-client-config but needs more effort to tease apart and ensure that
-        # values are being passed in. For now, let osc_lib do it and just set
-        # the values in occ.config
-        self._cli_options.config['verify'] = self.verify
-        self._cli_options.config['cacert'] = self.cacert
-        # Attack of the killer passthrough values
-        self._cli_options.config['cert'] = self._cli_options.cert
-        self._cli_options.config['key'] = self._cli_options.key
+        self.cacert = None
+        self.verify, self.cert = self._cli_options.get_requests_verify_args()
+        # If there is a cacert and we're verifying, it'll be in the verify
+        # argument.
+        if not isinstance(self.verify, bool):
+            self.cacert = self.verify
 
         # TODO(mordred) We also don't have any support for setting or passing
         # in api_timeout, which is set in occ defaults but we skip occ defaults
@@ -196,14 +167,10 @@ class ClientManager(object):
                 self._cli_options.remote_project_domain_name
             )
 
-        self.session = osc_session.TimingSession(
-            auth=self.auth,
-            verify=self.verify,
-            cert=self.cert,
-            app_name=self._app_name,
-            app_version=self._app_version,
-            additional_user_agent=[('osc-lib', version.version_string)],
-        )
+        self.session = self._cli_options.get_session()
+
+        self.sdk_connection = connection.Connection(config=self._cli_options)
+
         self._auth_setup_completed = True
 
     def validate_scope(self):
@@ -225,12 +192,25 @@ class ClientManager(object):
     @property
     def auth_ref(self):
         """Dereference will trigger an auth if it hasn't already"""
-        self.setup_auth()
-        return self.auth.get_access(self.session)
+        if (not self._auth_required or
+                self._cli_options.config['auth_type'] == 'none'):
+            # Forcibly skip auth if we know we do not need it
+            return None
+        if not self._auth_ref:
+            self.setup_auth()
+            LOG.debug("Get auth_ref")
+            self._auth_ref = self.auth.get_auth_ref(self.session)
+        return self._auth_ref
+
+    def _override_for(self, service_type):
+        key = '%s_endpoint_override' % service_type.replace('-', '_')
+        return self._cli_options.config.get(key)
 
     def is_service_available(self, service_type):
         """Check if a service type is in the current Service Catalog"""
-
+        # If there is an override, assume the service is available
+        if self._override_for(service_type):
+            return True
         # Trigger authentication necessary to discover endpoint
         if self.auth_ref:
             service_catalog = self.auth_ref.service_catalog
@@ -252,6 +232,11 @@ class ClientManager(object):
     def get_endpoint_for_service_type(self, service_type, region_name=None,
                                       interface='public'):
         """Return the endpoint URL for the service type."""
+        # Overrides take priority unconditionally
+        override = self._override_for(service_type)
+        if override:
+            return override
+
         if not interface:
             interface = 'public'
         # See if we are using password flow auth, i.e. we have a
